/**
 * @module direction/directioncommand
 */
import { Command } from 'ckeditor5/src/core';
import { first } from 'ckeditor5/src/utils';
import { isDefault } from './utils';

const DIRECTION = 'direction';

/**
 * The direction command plugin.
 */
export default class DirectionCommand extends Command {
  /**
   * @inheritDoc
   */
  refresh() {
    const firstBlock = first(
      this.editor.model.document.selection.getSelectedBlocks(),
    );
    // As first check whether to enable or disable the command as the value will always be false if the command cannot be enabled.
    this.isEnabled = !!firstBlock && this._canBeAligned(firstBlock);
    this.value = ( this.isEnabled && firstBlock.hasAttribute( 'direction' ) ) ? firstBlock.getAttribute( 'direction' ) : '';
  }

  /**
   * Executes the command.
   *
   * @param {Object} options
   *   An options object.
   * @param {String} options.value The new value of the `direction` attribute to set.
   */
  execute( options = {} ) {
    const { editor } = this;
    const { model } = editor;
    const doc = model.document;
    const value = options.value;

    model.change((writer) => {
      // Get only those blocks from selected that can have direction set
      const blocks = Array.from(doc.selection.getSelectedBlocks()).filter(
        (block) => this._canBeAligned(block),
      );
      const currentDirection = blocks[0].getAttribute('direction');

      // Remove direction attribute if current direction is:
      // - default (should not be stored in model as it will bloat model data)
      // - equal to currently set
      // - or no value is passed - denotes default direction.
      const removeDirection = isDefault( value ) || currentDirection === value || !value;

      if ( removeDirection ) {
        removeDirectionFromSelection( blocks, writer );
      } else {
        setDirectionOnSelection(blocks, writer, value);
      }
    });
  }

  /**
   * Checks whether a block can have direction set.
   *
   * @private
   *
   * @param {Element} block
   *   The block to check.
   * @return {Boolean}
   *   If block can be aligned.
   */
  _canBeAligned(block) {
    return this.editor.model.schema.checkAttribute(block, DIRECTION);
  }
}

/**
 * Removes the direction attribute from blocks.
 *
 * @private
 *
 * @param {Element[]} blocks
 *   The blocks to remove direction.
 * @param {Writer} writer
 *   The writer.
 */
function removeDirectionFromSelection(blocks, writer) {
  // eslint-disable-next-line no-restricted-syntax
  for (const block of blocks) {
    writer.removeAttribute(DIRECTION, block);
  }
}

/**
 * Sets the direction attribute on blocks.
 *
 * @private
 *
 * @param {Element[]} blocks
 *   The blocks to set.
 * @param {Writer} writer
 *   The writer.
 * @param {String} direction
 *   The direction to set to.
 */
function setDirectionOnSelection(blocks, writer, direction) {
  // eslint-disable-next-line no-restricted-syntax
  for (const block of blocks) {
    writer.setAttribute(DIRECTION, direction, block);
  }
}
