/**
 * @module direction/directionediting
 */

import { Plugin } from 'ckeditor5/src/core';
import DirectionCommand from './directioncommand';
import { isDefault, isSupported, supportedOptions } from './utils';

/**
 * The direction editing feature. It introduces the
 * {@link DirectionCommand command} and adds the `direction` attribute for
 * block elements.
 */
export default class DirectionEditing extends Plugin {

  /**
   * @inheritDoc
   */
  static get pluginName() {
    return 'DirectionEditing';
  }

  /**
   * @inheritDoc
   */
  constructor( editor ) {
    super( editor );

    editor.config.define( 'direction', {
      options: [ ...supportedOptions ]
    } );
  }

  /**
   * @inheritDoc
   */
  init() {
    const editor = this.editor;
    const schema = editor.model.schema;

    // Filter out unsupported options.
    const enabledOptions = editor.config.get( 'direction.options' ).filter( isSupported );

    // Allow direction attribute on all blocks.
    schema.extend( '$block', { allowAttributes: 'direction' } );
    editor.model.schema.setAttributeProperties( 'direction', { isFormatting: true } );

    const definition = _buildDefinition( enabledOptions.filter( option => !isDefault( option ) ) );

    editor.conversion.attributeToAttribute( definition );

    editor.commands.add( 'direction', new DirectionCommand( editor ) );
  }
}

/**
 * Checks whether a block can have direction set.
 *
 * @private
 *

 */
function _buildDefinition( options ) {
  const definition = {
    model: {
      key: 'direction',
      values: options.slice()
    },
    view: {}
  };

  for ( const option of options ) {
    definition.view[ option ] = {
      key: 'dir',
      value: option
    };
  }

  return definition;
}
