/**
 * @module direction/directionui
 */

import { Plugin } from 'ckeditor5/src/core';
import {
  ButtonView,
  createDropdown,
  addToolbarToDropdown,
} from 'ckeditor5/src/ui';
import { isSupported } from './utils';
import directionLeftIcon from '../../../../icons/ltr.svg';
import directionRightIcon from '../../../../icons/rtl.svg';

const icons = new Map([
  ['ltr', directionLeftIcon],
  ['rtl', directionRightIcon],
]);

/**
 * The default direction UI plugin.
 *
 * It introduces the buttons.
 */
export default class DirectionUI extends Plugin {

  /**
   * @inheritDoc
   */
  static get pluginName() {
    return 'DirectionUI';
  }

  /**
   * Returns the localized option titles provided by the plugin.
   *
   * @readonly
   * @type {Object.<String,String>}
   */
  get localizedOptionTitles() {
    const t = this.editor.t;

    return {
      'ltr': t('Direction left to right'),
      'rtl': t('Direction right to left'),
    };
  }

  /**
   * @inheritDoc
   */
  init() {
    const editor = this.editor;
    const componentFactory = editor.ui.componentFactory;
    const t = editor.t;
    const options = editor.config.get('direction.options');

    options
      .filter(isSupported)
      .forEach(option => this._addButton(option));

    componentFactory.add('direction', locale => {
      const dropdownView = createDropdown(locale);

      // Add existing direction buttons to dropdown's toolbar.
      const buttons = options.map(option => componentFactory.create(`direction:${option}`));
      addToolbarToDropdown(dropdownView, buttons,
        {
          enableActiveItemFocusOnDropdownOpen: true,
          isVertical: true,
          ariaLabel: t('Text direction toolbar'),
        });

      // Configure dropdown properties a behavior.
      dropdownView.buttonView.set({
        label: t('Text direction'),
        tooltip: true,
      });

      dropdownView.extendTemplate({
        attributes: {
          class: 'ck-direction-dropdown',
        },
      });

      // The default icon is aligned left as we do not support RTL yet (see #3).
      const defaultIcon = directionLeftIcon;

      // Change icon to reflect current selection's direction.
      dropdownView.buttonView.bind('icon').toMany(buttons, 'isOn', (...areActive) => {
        // Get the index of an active button.
        const index = areActive.findIndex(value => value);

        // If none of the commands is active, display either defaultIcon or the first button's icon.
        if (index < 0) {
          return defaultIcon;
        }

        // Return active button's icon.
        return buttons[index].icon;
      });

      // Enable button if any of the buttons is enabled.
      dropdownView.bind('isEnabled').toMany(buttons, 'isEnabled', (...areEnabled) => areEnabled.some(isEnabled => isEnabled));

      return dropdownView;
    });
  }

  /**
   * Helper method for initializing the button and linking it with an appropriate command.
   *
   * @private
   * @param {String} option The name of the direction option for which the button is added.
   */
  _addButton(option) {
    const editor = this.editor;

    editor.ui.componentFactory.add(`direction:${option}`, locale => {
      const command = editor.commands.get('direction');
      const buttonView = new ButtonView(locale);

      buttonView.set({
        label: this.localizedOptionTitles[option],
        icon: icons.get(option),
        tooltip: true,
      });

      // Bind button model to command.
      buttonView.bind('isEnabled').to(command);
      buttonView.bind('isOn').to(command, 'value', value => value === option);

      // Execute command.
      this.listenTo(buttonView, 'execute', () => {
        editor.execute('direction', { value: option });
        editor.editing.view.focus();
      });

      return buttonView;
    });
  }
}
